#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Indicator;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Strategy;
using TradingStudies.NinjaScript.Utility;
#endregion

namespace NinjaTrader.Strategy
{
    [Description("Scarlet Willow - A JMF3 Inception")]
    public class ATRTradeNQ : Strategy
    {
        #region Variables
        //DEFAULT SETTINGS FOR NQ 5 RANGE
		//Time Filter
        private int hourStart = 6; 		//My trading time; have not optimized for full day trading
        private int minuteStart = 30; 
        private int hourEnd = 8; 
        private int minuteEnd = 0;
		//P&L Filter
        private int maxWin = 1000;				
        private int maxLoss = 300; 			
		private int priorTradesCount = 0;
		private double priorTradesCumProfit = 0;
		//ATR Filter
		private bool bigATRFilter = false;
		private int bigATRPeriod = 107;			
		private double bigATRMultiplier = 5;  
		private int bigATRSmoothing = 21;		//use inhibits trading, but greatly increases in probabily; Multiplier = 6, Period = 9 (89% win rate Q1 2013)
		//ATR Trigger
		private int aTRPeriod = 14;          
		private double aTRMultiplier = 4.2; 	
		private int aTRSmoothing = 26;       	
		//CCIFilter
        private int cCIFilterPeriod = 74; 		
        private int cCIFilterLevel = 0; 
		private bool useCCIFilter = true;
		private bool useCCIOBOS = true;
		//Targets & Stops 
		private int target1=6;  				//5 maximizes win rate, 14 maximizes profit
		private int target2=46;					
		private int stop1=12;					//14 maximizes win rate, 10 or 12 maximizes profit, 9 also works
		private int quantity = 1;
		private int trailTicks = 1;				
		//Keltner Settings
		private int keltnerPeriod = 107;			
		private double keltnerMultiplier = 9.5;
		private bool useKeltnerExit = false;
		private bool useMAFilter = true;
		private int distanceToMA = 5;
		private bool midbandBias = true;
		//MA Settings
		private bool useMARisingFalling = true;
		private int mAPeriod = 67;
		//3BR
		private bool threeBRExit; 
		//Direction filter
		private bool longs = true;
		private bool shorts = true;
		//Re-Entry Trade
		private bool reEntry = false;
		private int reEntryTickDistance = 0;
		private bool useCCIReEntry = true;
		private bool use2BarEntry = true;
		//3BR Trade Entries - countertrend is best 
		private bool threeBR = true;
		private bool useMABiasFilter = false;
		private bool useDistanceToMAFilter = false;
		private bool useKeltnerFilter = true;		
		private int distanceToMA3BR = 5;
		private int distanceToKeltner = 12;			//1 maximizes win rate, but fairly infrequent
		private int target2BR = 17;
		//Basic Trend
		private bool basicTrend = true;
		//Box Trade
		private bool boxTrade = true;
		private int boxTradeTarget1 = 3;
		private int boxTradeTarget2 = 6;
		private int boxTradeStop = 12;
		//Pullback Pattern
		private bool pullbackPattern = true;
		//Filter
        #endregion
		
        protected override void Initialize()
        {
			#region Add Indicators, Set Targets, Entries Per Direction
            Add(TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR));
			
			
			//Longs Targets
            SetProfitTarget("LongBT1", CalculationMode.Ticks, target1);
			SetProfitTarget("LongBT2", CalculationMode.Ticks, target2);
			SetProfitTarget("RELong1", CalculationMode.Ticks, target1);
			SetProfitTarget("RELong2", CalculationMode.Ticks, target2);
			//Shorts Targets
            SetProfitTarget("ShortBT1", CalculationMode.Ticks, target1);
			SetProfitTarget("ShortBT2", CalculationMode.Ticks, target2);
			SetProfitTarget("REShort1", CalculationMode.Ticks, target1);
			SetProfitTarget("REShort2", CalculationMode.Ticks, target2);
			//3BR
			SetProfitTarget("3BRLong1",CalculationMode.Ticks,target1);
			SetProfitTarget("3BRLong2",CalculationMode.Ticks,target2BR);
			SetProfitTarget("3BRShort1",CalculationMode.Ticks,target1);
			SetProfitTarget("3BRShort2",CalculationMode.Ticks,target2BR);
			//Box Breakout
			SetProfitTarget("BoxLong1",CalculationMode.Ticks,boxTradeTarget1);
			SetProfitTarget("BoxLong2",CalculationMode.Ticks,boxTradeTarget2);
			SetProfitTarget("BoxShort1",CalculationMode.Ticks,boxTradeTarget1);
			SetProfitTarget("BoxShort2",CalculationMode.Ticks,boxTradeTarget2);
			//Dual Entries
            CalculateOnBarClose = true;
			EntriesPerDirection = 2; 
   			EntryHandling = EntryHandling.AllEntries;
			#endregion	
        }
        protected override void OnBarUpdate()
        {
			
			#region Performance Filter
			if (Bars.FirstBarOfSession)
			{// Store the strategy's prior cumulated realized profit and number of trades
				priorTradesCount = Performance.AllTrades.Count;
				priorTradesCumProfit = Performance.AllTrades.TradesPerformance.Currency.CumProfit;
			}
			// Stop trading if Win max or loss max is reached.
			if (Performance.AllTrades.TradesPerformance.Currency.CumProfit - priorTradesCumProfit >= maxWin
				|| Performance.AllTrades.TradesPerformance.Currency.CumProfit - priorTradesCumProfit <= -maxLoss)
				{return;}
			#endregion
				
			#region Targets, Stops and Trailing Stops	
			// Reset initial stop losses
			if (Position.MarketPosition == MarketPosition.Flat)
			{
				SetStopLoss("LongBT2",CalculationMode.Ticks,stop1,false);
				SetStopLoss("LongBT1",CalculationMode.Ticks,stop1,false);
				SetStopLoss("ShortBT1",CalculationMode.Ticks,stop1,false);
				SetStopLoss("ShortBT2",CalculationMode.Ticks,stop1,false);
				SetStopLoss("RELong1",CalculationMode.Ticks,stop1,false);
				SetStopLoss("RELong2",CalculationMode.Ticks,stop1,false);
				SetStopLoss("REShort1",CalculationMode.Ticks,stop1,false);
				SetStopLoss("REShort2",CalculationMode.Ticks,stop1,false);
				SetStopLoss("3BRLong1",CalculationMode.Ticks,stop1,false);
				SetStopLoss("3BRLong2",CalculationMode.Ticks,stop1,false);
				SetStopLoss("3BRShort1",CalculationMode.Ticks,stop1,false);
				SetStopLoss("3BRShort2",CalculationMode.Ticks,stop1,false);
				SetStopLoss("BoxLong1",CalculationMode.Ticks,boxTradeStop,false);
				SetStopLoss("BoxLong2",CalculationMode.Ticks,boxTradeStop,false);
				SetStopLoss("BoxShort1",CalculationMode.Ticks,boxTradeStop,false);
				SetStopLoss("BoxShort2",CalculationMode.Ticks,boxTradeStop,false);
			}
			// Set exit procedures if in a long position
			else if (Position.MarketPosition == MarketPosition.Long)
				{
					double lowerStop = TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend[0];			
					if (High[0] > Position.AvgPrice + target1 * TickSize)
						{SetStopLoss("LongBT2",CalculationMode.Price, Position.AvgPrice,false);
						 SetStopLoss("RELong2",CalculationMode.Price, Position.AvgPrice,false);
						 SetStopLoss("3BRLong2",CalculationMode.Price, Position.AvgPrice,false);
						 SetStopLoss("BoxLong2",CalculationMode.Price, Position.AvgPrice, false);}
					if(lowerStop>Position.AvgPrice)
						{SetStopLoss("LongBT2",CalculationMode.Price, lowerStop-trailTicks*TickSize,false);
						 SetStopLoss("RELong2",CalculationMode.Price, lowerStop-trailTicks*TickSize,false);
						 SetStopLoss("BoxLong2",CalculationMode.Price, lowerStop-trailTicks*TickSize,false);}
					if (useKeltnerExit == true && High[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Upper[0])
						{ExitLong("LongBT2");
						 ExitLong("RELong2");
						 ExitLong("3BRLong2");
						 ExitLong("BoxLong2");}
					if (High[1]>High[2]&&High[2]>High[3] && High[3]>High[4] && Low[1]>Low[2] && Low[2]>Low[3] && Low[3]>Low[4]
						&& Close[0]<Open[0]	&& Close[1]>Open[1] && Close[2]>Open[2] && Close[3]>Open[3]
						&& threeBRExit == true)
						{ExitLong("LongBT2");
						 ExitLong("RELong2");
						 ExitLong("3BRLong2");
						 ExitLong("BoxLong2");}
					if (useCCIOBOS == true && CCI(cCIFilterPeriod)[0]>=200)
						{ExitLong("LongBT2");
						 ExitLong("RELong2");
						 ExitLong("3BRLong2");
						 ExitLong("BoxLong2");}
						
				}	
			// Set exit procedures if in a short position
			else if (Position.MarketPosition == MarketPosition.Short)
				{
					double upperStop = TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend[0];
					if (Low[0] < Position.AvgPrice - target1 * TickSize)
						{SetStopLoss("ShortBT2",CalculationMode.Price, Position.AvgPrice,false);
						 SetStopLoss("REShort2",CalculationMode.Price, Position.AvgPrice,false);
						 SetStopLoss("3BRShort2",CalculationMode.Price, Position.AvgPrice,false);
						 SetStopLoss("BoxShort2",CalculationMode.Price, Position.AvgPrice, false);}
					if(upperStop<Position.AvgPrice)	
						{SetStopLoss("ShortBT2",CalculationMode.Price,upperStop + trailTicks*TickSize,false);
						 SetStopLoss("REShort2",CalculationMode.Price,upperStop + trailTicks*TickSize,false);
						 SetStopLoss("BoxShort2",CalculationMode.Price, upperStop + trailTicks*TickSize, false);}
					if (useKeltnerExit == true && Low[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Lower[0])
						{ExitShort("ShortBT2");
						 ExitShort("REShort2");
						 ExitShort("3BRShort2");
						 ExitShort("BoxShort2");}
					if (Low[1]<Low[2]&&Low[2]<Low[3]&&Low[3]<Low[4] && High[1]<High[2] && High[2]<High[3]
						&& High[3]<High[4]&&Close[0]>Open[0] && Close[1]<Open[1] && Close[2]<Open[2] && Close[3]<Open[3] 
						&& threeBRExit == true)
						{ExitShort("ShortBT2");
						 ExitShort("REShort2");
						 ExitShort("3BRShort2");
						 ExitShort("BoxShort2");}
					if (useCCIOBOS == true && CCI(cCIFilterPeriod)[0]<=-200)
						{ExitShort("ShortBT2");
						 ExitShort("REShort2");
						 ExitShort("3BRShort2");
						 ExitShort("BoxShort2");}
				}
			#endregion	
				
			#region Basic Trend Trades
			// Long entry trigger
			if (basicTrend == true)
			{
			if (CrossAbove(Close, TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend, 2)
				&& ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
				&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
				&& longs == true
				&& (useCCIFilter == false || CCI(CCIFilterPeriod)[0] > CCIFilterLevel)
				&& (useMAFilter == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] - distanceToMA*TickSize || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0]+2*TickSize)
				&& (midbandBias == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0])
				&& (bigATRFilter == false || Close[0]> TSSuperTrend(bigATRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, 
					bigATRMultiplier, bigATRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend[0])
				&& (useMARisingFalling == false || Rising(WMA(mAPeriod))))
			{
				EnterLong(quantity, "LongBT1");
				EnterLong(quantity,"LongBT2");
			}
			// Short entry trigger
			if (CrossBelow(Close, TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend, 2)
				&& (useCCIFilter == false || CCI(CCIFilterPeriod)[0] <= -CCIFilterLevel)
				&& ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
				&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
				&& shorts == true
				&& (useMAFilter == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] + distanceToMA*TickSize || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] + 2*TickSize)
				&& (midbandBias == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0])
				&& (bigATRFilter == false || Close[0]<TSSuperTrend(bigATRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA,
					bigATRMultiplier, bigATRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend[0])
				&& (useMARisingFalling == false || Falling(WMA(mAPeriod))))
			{
				EnterShort(quantity, "ShortBT1");
				EnterShort(quantity,"ShortBT2");
			}
			}
			#endregion
			
			#region Re-Entry Trend Trade
			//Long Re-Entry
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
				&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
				&& reEntry == true
				&& longs == true
				&& (use2BarEntry == true || Close[0]>Open[0] && Low[0]<=TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend[0]+reEntryTickDistance*TickSize
				&& High[0]>TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend[0]+reEntryTickDistance*TickSize
				&& Close[0]>TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend[0])
				&& (use2BarEntry == false || Close[1]>=Open[1] && Close[0]>=Open[0] && Low[1]<=TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend[1]+reEntryTickDistance*TickSize)
				&& (bigATRFilter == false || Close[0]> TSSuperTrend(bigATRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, 
					bigATRMultiplier, bigATRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend[0])
				&& (useCCIReEntry == false || CCI(CCIFilterPeriod)[0] > CCIFilterLevel)
				&& (useMAFilter == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] - distanceToMA*TickSize || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0]+2*TickSize)
				&& (midbandBias == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0]))
					{EnterLong(quantity,"RELong1");
					 EnterLong(quantity,"RELong2");}
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
				&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
				&& reEntry == true
				&& longs == true
				&& pullbackPattern == true
				&& (useCCIReEntry == false || CCI(CCIFilterPeriod)[0] > CCIFilterLevel)
				&& (useMAFilter == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] - distanceToMA*TickSize || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0]+2*TickSize)
				&& (midbandBias == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0])
				&& (Close[0]>Open[0] && Close[1]<Open[1] && Close[2]<Open[2] && Close[3]<Open[3] && Close[4]<Open[4] && Low[0]>=Low[3]
				|| Close[0]>Open[0] && Close[1]<Open[1] && Close[2]>Open[2] && Close[3]<Open[3] && Close[4]<Open[4] && Low[0]<Low[1] && Low[1]<Low[2] && Low[2]<Low[3]))
					{EnterLong(quantity,"RELong1");
					 EnterLong(quantity,"RELong2");}
				
			//Short Re-Entry
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
				&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
				&& reEntry == true
				&& shorts == true
				&& (use2BarEntry == true || Close[0]<Open[0] && High[0]>=TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend[0]-reEntryTickDistance*TickSize
				&& Low[0]<TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend[0]-reEntryTickDistance*TickSize
				&& Close[0]<TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend[0])
				&& (use2BarEntry == false || Close[1]<Open[1] && Close[0]<Open[0] && High[1]>=TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend[1]-reEntryTickDistance*TickSize)
				&& (bigATRFilter == false || Close[0]<TSSuperTrend(bigATRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA,
					bigATRMultiplier, bigATRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend[0])
				&& (useCCIReEntry == false || CCI(CCIFilterPeriod)[0] < CCIFilterLevel)
				&& (useMAFilter == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] + distanceToMA*TickSize || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] + 2*TickSize)
				&& (midbandBias == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0]))
					{EnterShort(quantity,"REShort1");
					 EnterShort(quantity,"REShort2");}
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
				&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
				&& reEntry == true
				&& shorts == true
				&& pullbackPattern == true
				&& (useCCIReEntry == false || CCI(CCIFilterPeriod)[0] < CCIFilterLevel)
				&& (useMAFilter == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] + distanceToMA*TickSize || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] + 2*TickSize)
				&& (midbandBias == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0])
				&& Close[0]<Open[0] && Close[1]>Open[1] && Close[2]<Open[2] && Close[3]>Open[3] && Close[4]>Open[4] && High[0]<=High[3])
					{EnterShort(quantity,"REShort1");
					 EnterShort(quantity,"REShort2");}
			#endregion
					
			#region 3BR Trade
			//3BR Long
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
						&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
						&& Low[1]<Low[2]&&Low[2]<Low[3]&&Low[3]<Low[4] && High[1]<High[2] && High[2]<High[3]
						&& High[3]<High[4]&&Close[0]>Open[0] && Close[1]<Open[1] && Close[2]<Open[2] && Close[3]<Open[3]
						&& (useMABiasFilter == false || Low[1]>= SMA(keltnerPeriod)[1])
						&& (useDistanceToMAFilter == false || Low[1]<= WMA(mAPeriod)[1]+distanceToMA3BR*TickSize)
						&& (useKeltnerFilter == false || Low[1]<=KeltnerChannel(keltnerMultiplier,keltnerPeriod).Lower[0]+distanceToKeltner*TickSize)
						&& threeBR == true
						&& longs == true)
				{EnterLong(quantity,"3BRLong1");
				 EnterLong(quantity,"3BRLong2");}
			//3BR Short
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
						&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
						&& High[1]>High[2]&&High[2]>High[3] && High[3]>High[4] && Low[1]>Low[2] && Low[2]>Low[3] && Low[3]>Low[4]
						&& Close[0]<Open[0]	&& Close[1]>Open[1] && Close[2]>Open[2] && Close[3]>Open[3]
						&& (useMABiasFilter == false || High[1]<= SMA(keltnerPeriod)[1])
						&& (useDistanceToMAFilter == false || High[1]>= WMA(mAPeriod)[1]-distanceToMA3BR*TickSize)
						&& (useKeltnerFilter == false || High[1]>=KeltnerChannel(keltnerMultiplier,keltnerPeriod).Upper[0]-distanceToKeltner*TickSize)
						&& threeBR == true
						&& shorts == true)
				{EnterShort(quantity,"3BRShort1");
				 EnterShort(quantity,"3BRShort2");}
				#endregion		
				
			#region Box Breakout Trade
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
						&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
						&& High[0]<High[3]+2*TickSize && High[0]<High[2]+2*TickSize && High[0]<High[1]+2*TickSize && Low[0]>Low[3]-2*TickSize
						&& Low[0]>Low[2]-2*TickSize && Low[0]>Low[1]-2*TickSize
						&& boxTrade == true
						&& longs == true
						&& (midbandBias == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0])
						&& (useCCIFilter == false || CCI(CCIFilterPeriod)[0] > CCIFilterLevel)
						&& (useMARisingFalling == false || Rising(WMA(mAPeriod))))
				{
					double range = MAX(Highs[0],3)[0]-MIN(Lows[0],3)[0];
					double orderLocation = MAX(Highs[0],3)[0] + range/2;
					EnterLongStop(DefaultQuantity,orderLocation,"BoxLong1");
					EnterLongStop(DefaultQuantity,orderLocation,"BoxLong2");
				}
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
						&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
						&& boxTrade == true
						&& longs == true
						&& (midbandBias == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0])
						&& (useCCIFilter == false || CCI(CCIFilterPeriod)[0] > CCIFilterLevel)
						&& (useMARisingFalling == false || Rising(WMA(mAPeriod)))
						&& High[1]<High[4]+2*TickSize && High[1]<High[3]+2*TickSize && High[1]<High[2]+2*TickSize && Low[1]>Low[4]-2*TickSize
						&& Low[1]>Low[3]-2*TickSize && Low[1]>Low[2]-2*TickSize
						&& Position.MarketPosition == MarketPosition.Flat)
					{
						double range = MAX(Highs[0],3)[1]-MIN(Lows[0],3)[1];
						double orderLocation = MAX(Highs[0],3)[1] + range/2;
						EnterLongStop(DefaultQuantity,orderLocation,"BoxLong1");
						EnterLongStop(DefaultQuantity,orderLocation,"BoxLong2");
					}
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
						&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
						&& boxTrade == true
						&& longs == true
						&& (midbandBias == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0])
						&& (useCCIFilter == false || CCI(CCIFilterPeriod)[0] > CCIFilterLevel)
						&& (useMARisingFalling == false || Rising(WMA(mAPeriod)))
						&& High[2]<High[5]+2*TickSize && High[2]<High[4]+2*TickSize && High[2]<High[3]+2*TickSize && Low[2]>Low[5]-2*TickSize
						&& Low[2]>Low[4]-2*TickSize && Low[2]>Low[5]-2*TickSize
						&& Position.MarketPosition == MarketPosition.Flat)
					{
						double range = MAX(Highs[0],3)[2]-MIN(Lows[0],3)[2];
						double orderLocation = MAX(Highs[0],3)[2] + range/2;
						EnterLongStop(DefaultQuantity,orderLocation,"BoxLong1");
						EnterLongStop(DefaultQuantity,orderLocation,"BoxLong2");
					}	
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
						&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
						&& High[0]<High[3]+2*TickSize && High[0]<High[2]+2*TickSize && High[0]<High[1]+2*TickSize && Low[0]>Low[3]-2*TickSize
						&& Low[0]>Low[2]-2*TickSize && Low[0]>Low[1]-2*TickSize
				 		&& boxTrade == true
						&& shorts == true
						&& (midbandBias == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0])
						&& (useCCIFilter == false || CCI(CCIFilterPeriod)[0] < CCIFilterLevel)
						&& (useMARisingFalling == false || Falling(WMA(mAPeriod))))
				{
					double range = MAX(Highs[0],3)[0]-MIN(Lows[0],3)[0];
					double orderLocationShort = MIN(Lows[0],3)[0] - range/2;
					EnterShortStop(DefaultQuantity, orderLocationShort,"BoxShort1");
					EnterShortStop(DefaultQuantity, orderLocationShort,"BoxShort2");
				}	
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
						&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
						&& High[1]<High[4]+2*TickSize && High[1]<High[3]+2*TickSize && High[1]<High[2]+2*TickSize && Low[1]>Low[4]-2*TickSize
						&& Low[1]>Low[3]-2*TickSize && Low[1]>Low[2]-2*TickSize
				 		&& boxTrade == true
						&& shorts == true
						&& Position.MarketPosition == MarketPosition.Flat
						&& (midbandBias == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0])
						&& (useCCIFilter == false || CCI(CCIFilterPeriod)[0] < CCIFilterLevel)
						&& (useMARisingFalling == false || Falling(WMA(mAPeriod))))
				{
					double range = MAX(Highs[0],3)[1]-MIN(Lows[0],3)[1];
					double orderLocationShort = MIN(Lows[0],3)[1] - range/2;
					EnterShortStop(DefaultQuantity, orderLocationShort,"BoxShort1");
					EnterShortStop(DefaultQuantity, orderLocationShort,"BoxShort2");
				}
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
						&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
						&& High[2]<High[5]+2*TickSize && High[2]<High[4]+2*TickSize && High[2]<High[3]+2*TickSize && Low[2]>Low[5]-2*TickSize
						&& Low[2]>Low[4]-2*TickSize && Low[2]>Low[5]-2*TickSize
				 		&& boxTrade == true
						&& shorts == true
						&& Position.MarketPosition == MarketPosition.Flat
						&& (midbandBias == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0])
						&& (useCCIFilter == false || CCI(CCIFilterPeriod)[0] < CCIFilterLevel)
						&& (useMARisingFalling == false || Falling(WMA(mAPeriod))))
				{
					double range = MAX(Highs[0],3)[2]-MIN(Lows[0],3)[2];
					double orderLocationShort = MIN(Lows[0],3)[2] - range/2;
					EnterShortStop(DefaultQuantity, orderLocationShort,"BoxShort1");
					EnterShortStop(DefaultQuantity, orderLocationShort,"BoxShort2");
				}
			#endregion
		}
        #region Properties
		
		#region A.Time
        [Description("Start Hour")]
        [GridCategory("A.Time")]
        public int StartHour
        {
            get { return hourStart; }
            set { hourStart = Math.Max(1, value); }
        }
        [Description("Start Minute")]
        [GridCategory("A.Time")]
        public int StartMinute
        {
            get { return minuteStart; }
            set { minuteStart = Math.Max(0, value); }
        }
        [Description("End Hour")]
        [GridCategory("A.Time")]
        public int StopHour
        {
            get { return hourEnd; }
            set { hourEnd = Math.Max(1, value); }
        }
        [Description("End Minute")]
        [GridCategory("A.Time")]
        public int StopMinute
        {
            get { return minuteEnd; }
            set { minuteEnd = Math.Max(0, value); }
        }
		#endregion
		
		#region B.Daily Profit & Loss Limit
		//P & L, Targets & Stops Publics
        [Description("Max Daily Profit Limit")]
        [GridCategory("B.P&L Limits")]
        public int MaxWin
        {
            get { return maxWin; }
            set { maxWin = Math.Max(1, value); }
        }
        [Description("Max Daily Loss Limit")]
        [GridCategory("B.P&L Limits")]
        public int MaxLoss
        {
            get { return maxLoss; }
            set { maxLoss = Math.Max(0, value); }
        }
		#endregion
		
		#region C.Targets & Stops
		[Description("Trade Quantity")]
        [GridCategory("C.Targets & Stops")]
        public int Quantity
        {
            get { return quantity; }
            set { quantity = Math.Max(1, value); }
        }
		 [Description("First Contract Profit Target")]
        [GridCategory("C.Targets & Stops")]
        public int Target1
        {
            get { return target1; }
            set { target1 = Math.Max(1, value); }
        }
        [Description("Second Contract Profit Target")]
        [GridCategory("C.Targets & Stops")]
        public int Target2
        {
            get { return target2; }
            set { target2 = Math.Max(1, value); }
        }
		[Description("Trade Stop")]
        [GridCategory("C.Targets & Stops")]
        public int Stop1
        {
            get { return stop1; }
            set { stop1 = Math.Max(1, value); }
		}
		[Description("Ticks to Trail ATR Stop")]
        [GridCategory("C.Targets & Stops")]
        public int TicksToTrail
        {
            get { return trailTicks; }
            set { trailTicks = Math.Max(0, value); }
        }
		#endregion
		
		#region D.Directional Filter
		[Description("Long Trades On")]
        [GridCategory("D.Directional Filter")]
        public bool LongTradesOn
        {
            get { return longs; }
            set { longs = value; }
        }
		[Description("Short Trades On")]
        [GridCategory("D.Directional Filter")]
        public bool ShortTradesOn
        {
            get { return shorts; }
            set { shorts = value; }
        }
		#endregion
		
		#region E.Trade Types
		[Description("Re-entry Trade from ATR")]
        [GridCategory("E.Trade Types")]
        public bool ReEntry
        {
            get { return reEntry; }
            set { reEntry = value; }
        }
		[Description("Basic trend trade with selected filters")]
        [GridCategory("E.Trade Types")]
        public bool BasicTrend
        {
            get { return basicTrend; }
            set { basicTrend = value; }
        }
		[Description("Lower high / higher low pullback pattern - must have Re-Entry trade enable for this trade to fire.")]
        [GridCategory("E.Trade Types")]
        public bool PullbackPattern
        {
            get { return pullbackPattern; }
            set { pullbackPattern = value; }
        }
		[Description("Three bar reversal (3BR) trades - can be countertrend (use Keltner outerband filter) or with trend only (use MA or ATR filters)")]
        [GridCategory("E.Trade Types")]
        public bool ThreeBRTrades
        {
            get { return threeBR; }
            set { threeBR = value; }
        } 
		[Description("Turn Box breakout trades on or off")]
        [GridCategory("E.Trade Types")]
        public bool BoxTrades
        {
            get { return boxTrade; }
            set { boxTrade = value; }
        }
		#endregion
		
		#region F.Trigger ATR Settings
		[Description("ATR Period")]
        [GridCategory("F.ATR Trigger Settings")]
        public int ATRPeriod
        {
            get { return aTRPeriod; }
            set { aTRPeriod = Math.Max(1, value); }
        }
		
		[Description("ATR Multiplier")]
        [GridCategory("F.ATR Trigger Settings")]
        public double ATRMultiplier
        {
            get { return aTRMultiplier; }
            set { aTRMultiplier = Math.Max(1, value); }
        }
		
		[Description("ATR Smoothing")]
        [GridCategory("F.ATR Trigger Settings")]
        public int ATRSmoothing
        {
            get { return aTRSmoothing; }
            set { aTRSmoothing = Math.Max(1, value); }
        }
		#endregion
		
		#region G.Filter ATR Settings
		[Description("ATR Period")]
        [GridCategory("G.ATR Filter Settings")]
        public int FilterATRPeriod
        {
            get { return bigATRPeriod; }
            set { bigATRPeriod = Math.Max(1, value); }
        }
		
		[Description("ATR Multiplier")]
        [GridCategory("G.ATR Filter Settings")]
        public double FilterATRMultiplier
        {
            get { return bigATRMultiplier; }
            set { bigATRMultiplier = Math.Max(1, value); }
        }
		
		[Description("ATR Smoothing")]
        [GridCategory("G.ATR Filter Settings")]
        public int FilterATRSmoothing
        {
            get { return bigATRSmoothing; }
            set { bigATRSmoothing = Math.Max(1, value); }
        }
		
		[Description("Use HTF ATR Filter")]
        [GridCategory("G.ATR Filter Settings")]
        public bool UseATRFilter
        {
            get { return bigATRFilter; }
            set { bigATRFilter = value; }
        }
		#endregion
		
		#region H.CCI Settings
        [Description("CCI Period")]
        [GridCategory("H.CCI Settings")]
        public int CCIFilterPeriod
        {
            get { return cCIFilterPeriod; }
            set { cCIFilterPeriod = Math.Max(1, value); }
        }
        [Description("CCI Filter Level")]
        [GridCategory("H.CCI Settings")]
        public int CCIFilterLevel
        {
            get { return cCIFilterLevel; }
            set { cCIFilterLevel = Math.Max(0, value); }
        }
		[Description("Use CCI Filter")]
        [GridCategory("H.CCI Settings")]
        public bool UseCCIFilter
        {
            get { return useCCIFilter; }
            set { useCCIFilter = value; }
        }
		[Description("Use CCI Overbought or Oversol to exit trade")]
        [GridCategory("H.CCI Settings")]
        public bool UseCCIOBOS
        {
            get { return useCCIOBOS; }
            set { useCCIOBOS = value; }
        }
		#endregion
	
		#region I.Keltner Settings
		[Description("Keltner Period")]
        [GridCategory("I.Keltner Settings")]
        public int KeltnerPeriod
        {
            get { return keltnerPeriod; }
            set { keltnerPeriod = Math.Max(1, value); }
        }
		[Description("Keltner Exit")]
        [GridCategory("I.Keltner Settings")]
        public bool UseKeltnerExit
        {
            get { return useKeltnerExit; }
            set { useKeltnerExit = value; }
        }
		
		[Description("Distance to Midband Filter")]
        [GridCategory("I.Keltner Settings")]
        public int DistanceToMidband
        {
            get { return distanceToMA; }
            set { distanceToMA = Math.Max(1, value); }
        }
		
		[Description("Use Midband Filter")]
        [GridCategory("I.Keltner Settings")]
        public bool UseMidbandFilter
        {
            get { return useMAFilter; }
            set { useMAFilter = value; }
        }
        [Description("Keltner Multiplier")]
        [GridCategory("I.Keltner Settings")]
        public double KeltnerMultiplier
        {
            get { return keltnerMultiplier; }
            set { keltnerMultiplier = Math.Max(0, value); }
        }
		
		[Description("Take Trades only on correct side of Midband")]
        [GridCategory("I.Keltner Settings")]
        public bool MidbandBias
        {
            get { return midbandBias; }
            set { midbandBias = value; }
        }
		#endregion
		
		#region J.Moving Average
		[Description("Using moving average slope as a bias for trades")]
        [GridCategory("J.Moving Average")]
        public bool UseMASlope
        {
            get { return useMARisingFalling; }
            set { useMARisingFalling = value; }
        }
		[Description("Moving average period")]
        [GridCategory("J.Moving Average")]
        public int MovingAveragePeriod
        {
            get { return mAPeriod; }
            set { mAPeriod = Math.Max(1, value); }
        }
		#endregion
		
		#region K.3BR Trade Settings
		[Description("Pullback must be a certain distance from the moving average (in ticks) to look for 3BR trades")]
        [GridCategory("K.3BR")]
        public bool UseDistanceToMAFilter
        {
            get { return useDistanceToMAFilter; }
            set { useDistanceToMAFilter = value; }
        }
		[Description("3BR must be within x ticks of the moving average to take the 3BR trade")]
        [GridCategory("K.3BR")]
        public int DistanceToMA3BR
        {
            get { return distanceToMA3BR; }
            set { distanceToMA3BR = Math.Max(1, value); }
        }
		[Description("Price must be within x ticks or beyond Keltner outerband for a countertrend reversal trade")]
        [GridCategory("K.3BR")]
        public bool UseKeltnerFilter
        {
            get { return useKeltnerFilter; }
            set { useKeltnerFilter = value; }
        }
		[Description("3BR must be within x ticks from the Keltner outerband to take the 3BR countertrend trade")]
        [GridCategory("K.3BR")]
        public int DistanceToKeltner
        {
            get { return distanceToKeltner; }
            set { distanceToKeltner = Math.Max(1, value); }
        }
		[Description("Target for Second Contract on 3BR Trades")]
        [GridCategory("K.3BR")]
        public int Target2for3BR
        {
            get { return target2BR; }
            set { target2BR = Math.Max(1,value); }
        }
		[Description("Three Bar Reversal as Exit")]
        [GridCategory("K.3BR")]
        public bool ThreeBarReversalExit
        {
            get { return threeBRExit; }
            set { threeBRExit = value; }
        }
		[Description("Using above/below Keltner Midband as a bias for 3BR trades - pullback trades only")]
        [GridCategory("K.3BR")]
        public bool UseMidBandBias
        {
            get { return useMABiasFilter; }
            set { useMABiasFilter = value; }
        }
		#endregion
		
		#region L.Box Trade Settings
		[Description("First target for box trades")]
        [GridCategory("L.Box Trade")]
        public int BoxTradeTarget1
        {
            get { return boxTradeTarget1; }
            set { boxTradeTarget1 = Math.Max(1,value); }
        }
		[Description("Second target for box trades")]
        [GridCategory("L.Box Trade")]
        public int BoxTradeTarget2
        {
            get { return boxTradeTarget2; }
            set { boxTradeTarget2 = Math.Max(1,value); }
        }
		[Description("Stop loss for box trades")]
        [GridCategory("L.Box Trade")]
        public int BoxTradeStop
        {
            get { return boxTradeStop; }
            set { boxTradeStop = Math.Max(1,value); }
        }
		#endregion
		
		#region M.Re-Entry Trade Settings
		[Description("Re-entry Trade from ATR requires two closes in direction of trade prior to entry")]
        [GridCategory("M.Re-Entry Trade")]
        public bool Use2BarEntry
        {
            get { return use2BarEntry; }
            set { use2BarEntry = value; }
        }
		[Description("Distance from Trigger ATR for Re-Entry Trade")]
        [GridCategory("M.Re-Entry Trade")]
        public int ReEntryTickDistance
        {
            get { return reEntryTickDistance; }
            set { reEntryTickDistance = Math.Max(0, value); }
        }
		[Description("Use CCI Filter for ReEntry trades")]
        [GridCategory("M.Re-Entry Trade")]
        public bool UseCCIforReEntry
        {
            get { return useCCIReEntry; }
            set { useCCIReEntry = value; }
        }
		#endregion
		
        #endregion
    }
}
